@@ -8,7 +8,7 @@
 #' \preformatted{
 #' .$set(key, value)
 #' .$get(key, default)
-#' .$remove(key)
+#' .$remove(key, silent = FALSE)
 #' .$pop(key, default)
 #' .$has(key)
 #' .$keys()
@@ -46,6 +46,7 @@
 #' @export
 dict <- function(items = NULL, keys = NULL) {
     self <- environment()
+    .__class__ <- "dict"
 
     n <- NULL
     m <- NULL
@@ -54,7 +55,6 @@ dict <- function(items = NULL, keys = NULL) {
     vs <- NULL
     ks <- NULL
     ht_xptr <- NULL
-    digest <- digest::digest
     # we will define the keys function
     .keys <- keys
 
@@ -81,15 +81,16 @@ dict <- function(items = NULL, keys = NULL) {
         invisible(self)
     }
     get <- function(key, default) {
-        .Call(C_dict_get, self, key, missing_arg(default))
+        .Call(C_dict_get, self, key)
     }
-    remove <- function(key) {
-        .Call(C_dict_remove, self, key)
+    remove <- function(key, silent = FALSE) {
+        .Call(C_dict_remove, self, key, silent)
         invisible(self)
     }
     pop <- function(key, default) {
+        # TODO: get and remove in one shot
         v <- get(key, default)
-        remove(key)
+        .Call(C_dict_remove, self, key, TRUE)
         v
     }
     has <- function(key) {
