@@ -1,10 +1,9 @@
 #include "xxh.h"
 
-
 int is_hashable(SEXP key) {
     if (Rf_isNull(key)) {
         return 1;
-    }else if (Rf_isVectorAtomic(key)) {
+    } else if (Rf_isVectorAtomic(key)) {
         if (!is_hashable(ATTRIB(key))) {
             return 0;
         }
@@ -12,10 +11,14 @@ int is_hashable(SEXP key) {
     } else if (TYPEOF(key) == VECSXP) {
         R_xlen_t i;
         R_xlen_t n = Rf_length(key);
+        SEXP keyi;
         for (i = 0; i < n; i++) {
-            if (!is_hashable(VECTOR_ELT(key, i))) {
+            keyi = PROTECT(VECTOR_ELT(key, i));
+            if (!is_hashable(keyi)) {
+                UNPROTECT(1);
                 return 0;
             }
+            UNPROTECT(1);
         }
         if (!is_hashable(ATTRIB(key))) {
             return 0;
@@ -38,23 +41,21 @@ int is_hashable(SEXP key) {
     return 0;
 }
 
-
 // much of the following is derived from the fastdigest package but adapt to xxh
 
 static char* buf1;
 
 static void OutChar(R_outpstream_t stream, int c) {
-    XXH3_state_t* const xxh_state = (XXH3_state_t* const) stream->data;
-    buf1[0] = (char) c;
+    XXH3_state_t* const xxh_state = (XXH3_state_t* const)stream->data;
+    buf1[0] = (char)c;
     XXH3_64bits_update(xxh_state, buf1, 1);
 }
 
-static void OutBytes(R_outpstream_t stream, void *buf, int length) {
-    XXH3_state_t* const xxh_state = (XXH3_state_t* const) stream->data;
+static void OutBytes(R_outpstream_t stream, void* buf, int length) {
+    XXH3_state_t* const xxh_state = (XXH3_state_t* const)stream->data;
     XXH3_64bits_update(xxh_state, buf, length);
 }
 
-
 XXH64_hash_t xxh_serialized_digest(SEXP x) {
     XXH3_state_t* const xxh_state = XXH3_createState();
     XXH3_64bits_reset(xxh_state);
@@ -63,8 +64,7 @@ XXH64_hash_t xxh_serialized_digest(SEXP x) {
     int version = 2;
 
     buf1 = malloc(1);
-    R_InitOutPStream(&stream, (R_pstream_data_t) xxh_state, type, version,
-                 OutChar, OutBytes, NULL, R_NilValue);
+    R_InitOutPStream(&stream, (R_pstream_data_t)xxh_state, type, version, OutChar, OutBytes, NULL, R_NilValue);
 
     R_Serialize(x, &stream);
 
@@ -74,21 +74,20 @@ XXH64_hash_t xxh_serialized_digest(SEXP x) {
     return res;
 }
 
-
 XXH64_hash_t xxh_digest(SEXP x) {
     if (Rf_length(x) >= 0 && Rf_isVectorAtomic(x)) {
         // note: always materialize ALTREP
-        char *p;
+        char* p;
         if (TYPEOF(x) == STRSXP) {
             if (Rf_length(x) == 1) {
-                p = (char *) Rf_translateCharUTF8(Rf_asChar(x));
+                p = (char*)Rf_translateCharUTF8(Rf_asChar(x));
                 return XXH3_64bits(p, strlen(p));
             } else {
                 XXH3_state_t* const xxh_state = XXH3_createState();
                 XXH3_64bits_reset(xxh_state);
                 R_xlen_t n = Rf_length(x);
                 for (R_xlen_t i = 0; i < n; i++) {
-                    p = (char *) Rf_translateCharUTF8(STRING_ELT(x, i));
+                    p = (char*)Rf_translateCharUTF8(STRING_ELT(x, i));
                     XXH3_64bits_update(xxh_state, p, strlen(p));
                 }
                 XXH64_hash_t res = XXH3_64bits_digest(xxh_state);
@@ -97,19 +96,19 @@ XXH64_hash_t xxh_digest(SEXP x) {
             }
         }
         if (TYPEOF(x) == INTSXP) {
-            p = (char*) INTEGER(x);
+            p = (char*)INTEGER(x);
             return XXH3_64bits(p, Rf_length(x) * sizeof(int));
         }
         if (TYPEOF(x) == REALSXP) {
-            p = (char*) REAL(x);
+            p = (char*)REAL(x);
             return XXH3_64bits(p, Rf_length(x) * sizeof(double));
         }
         if (TYPEOF(x) == LGLSXP) {
-            p = (char*) LOGICAL(x);
+            p = (char*)LOGICAL(x);
             return XXH3_64bits(p, Rf_length(x) * sizeof(int));
         }
         if (TYPEOF(x) == RAWSXP) {
-            p = (char*) RAW(x);
+            p = (char*)RAW(x);
             return XXH3_64bits(p, Rf_length(x));
         }
     }
